@@ -1,5 +1,4 @@
-# -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2012 Bastian Kleineidam
+# Copyright (C) 2000-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,20 +18,21 @@ Main functions for link checking.
 """
 
 import os
-import cgi
-import logging
-import urllib
-from .. import strformat, url as urlutil, log, LOG_CHECK
+import html
+import urllib.parse
 
-MAX_FILESIZE = 1024*1024*10 # 10MB
+from .. import url as urlutil, log, LOG_CHECK
+
+MAX_FILESIZE = 1024 * 1024 * 10  # 10MB
 
 
 def guess_url(url):
     """Guess if URL is a http or ftp URL.
+
     @param url: the URL to check
-    @ptype url: unicode
+    @type url: unicode
     @return: url with http:// or ftp:// prepended if it's detected as
-      a http respective ftp URL.
+        a http respective ftp URL.
     @rtype: unicode
     """
     if url.lower().startswith("www."):
@@ -44,7 +44,7 @@ def guess_url(url):
     return url
 
 
-def absolute_url (base_url, base_ref, parent_url):
+def absolute_url(base_url, base_ref, parent_url):
     """
     Search for the absolute url to detect the link type. This does not
     join any url fragments together!
@@ -62,12 +62,23 @@ def absolute_url (base_url, base_ref, parent_url):
         return base_ref
     elif parent_url and urlutil.url_is_absolute(parent_url):
         return parent_url
-    return u""
+    return ""
 
 
-def get_url_from (base_url, recursion_level, aggregate,
-                  parent_url=None, base_ref=None, line=0, column=0,
-                  name=u"", parent_content_type=None, extern=None):
+def get_url_from(
+    base_url,
+    recursion_level,
+    aggregate,
+    parent_url=None,
+    base_ref=None,
+    line=None,
+    column=None,
+    page=0,
+    name="",
+    parent_content_type=None,
+    extern=None,
+    url_encoding=None,
+):
     """
     Get url data from given base data.
 
@@ -80,63 +91,81 @@ def get_url_from (base_url, recursion_level, aggregate,
     @param parent_url: parent url
     @type parent_url: string or None
     @param base_ref: base url from <base> tag
-    @type base_ref string or None
+    @type base_ref: string or None
     @param line: line number
     @type line: number
     @param column: column number
     @type column: number
+    @param page: page number
+    @type page: number
     @param name: link name
     @type name: string
     @param extern: (is_extern, is_strict) or None
     @type extern: tuple(int, int) or None
     """
     if base_url is not None:
-        base_url = strformat.unicode_safe(base_url)
         # left strip for detection of URL scheme
         base_url_stripped = base_url.lstrip()
     else:
         base_url_stripped = base_url
-    if parent_url is not None:
-        parent_url = strformat.unicode_safe(parent_url)
-    if base_ref is not None:
-        base_ref = strformat.unicode_safe(base_ref)
-    name = strformat.unicode_safe(name)
     url = absolute_url(base_url_stripped, base_ref, parent_url).lower()
-    if not (url or name):
-        # use filename as base url, with slash as path seperator
-        name = base_url.replace("\\", "/")
-    if parent_content_type == 'application/x-httpd-php' and \
-       '<?' in base_url and '?>' in base_url and url.startswith('file:'):
-        # ignore but warn about URLs from local PHP files with execution directives
+    if ":" in url:
+        scheme = url.split(":", 1)[0].lower()
+    else:
+        scheme = None
+        if not (url or name):
+            # use filename as base url, with slash as path seperator
+            name = base_url.replace("\\", "/")
+    allowed_schemes = aggregate.config["allowedschemes"]
+    # ignore local PHP files with execution directives
+    local_php = (
+        parent_content_type == 'application/x-httpd-php'
+        and '<?' in base_url
+        and '?>' in base_url
+        and scheme == 'file'
+    )
+    if local_php or (allowed_schemes and scheme not in allowed_schemes):
         klass = ignoreurl.IgnoreUrl
     else:
         assume_local_file = recursion_level == 0
-        klass = get_urlclass_from(url, assume_local_file=assume_local_file)
+        klass = get_urlclass_from(scheme, assume_local_file=assume_local_file)
     log.debug(LOG_CHECK, "%s handles url %s", klass.__name__, base_url)
-    return klass(base_url, recursion_level, aggregate,
-                 parent_url=parent_url, base_ref=base_ref,
-                 line=line, column=column, name=name, extern=extern)
+    return klass(
+        base_url,
+        recursion_level,
+        aggregate,
+        parent_url=parent_url,
+        base_ref=base_ref,
+        line=line,
+        column=column,
+        page=page,
+        name=name,
+        extern=extern,
+        url_encoding=url_encoding,
+    )
 
 
-def get_urlclass_from (url, assume_local_file=False):
-    """Return checker class for given URL. If URL does not start
-    with a URL scheme and assume_local_file is True, assume that
-    the given URL is a local file."""
-    if url.startswith("http:"):
+def get_urlclass_from(scheme, assume_local_file=False):
+    """Return checker class for given URL scheme. If the scheme
+    cannot be matched and assume_local_file is True, assume a local file.
+    """
+    if scheme in ("http", "https"):
         klass = httpurl.HttpUrl
-    elif url.startswith("ftp:"):
+    elif scheme == "ftp":
         klass = ftpurl.FtpUrl
-    elif url.startswith("file:"):
+    elif scheme == "file":
         klass = fileurl.FileUrl
-    elif url.startswith("telnet:"):
+    elif scheme == "telnet":
         klass = telneturl.TelnetUrl
-    elif url.startswith("mailto:"):
+    elif scheme == "mailto":
         klass = mailtourl.MailtoUrl
-    elif url.startswith("https:"):
-        klass = httpsurl.HttpsUrl
-    elif url.startswith(("nntp:", "news:", "snews:")):
+    elif scheme in ("nntp", "news", "snews"):
         klass = nntpurl.NntpUrl
-    elif unknownurl.is_unknown_url(url):
+    elif scheme == "dns":
+        klass = dnsurl.DnsUrl
+    elif scheme == "itms-services":
+        klass = itmsservicesurl.ItmsServicesUrl
+    elif scheme and unknownurl.is_unknown_scheme(scheme):
         klass = unknownurl.UnknownUrl
     elif assume_local_file:
         klass = fileurl.FileUrl
@@ -145,7 +174,7 @@ def get_urlclass_from (url, assume_local_file=False):
     return klass
 
 
-def get_index_html (urls):
+def get_index_html(urls):
     """
     Construct artificial index.html from given URLs.
 
@@ -154,9 +183,9 @@ def get_index_html (urls):
     """
     lines = ["<html>", "<body>"]
     for entry in urls:
-        name = cgi.escape(entry)
+        name = html.escape(entry)
         try:
-            url = cgi.escape(urllib.quote(entry))
+            url = html.escape(urllib.parse.quote(entry))
         except KeyError:
             # Some unicode entries raise KeyError.
             url = name
@@ -165,24 +194,16 @@ def get_index_html (urls):
     return os.linesep.join(lines)
 
 
-class StoringHandler (logging.Handler):
-    """Store all emitted log messages in a size-limited list.
-    Used by the CSS syntax checker."""
-
-    def __init__ (self, maxrecords=100):
-        """Initialize site-limited list."""
-        logging.Handler.__init__(self)
-        self.storage = []
-        self.maxrecords = maxrecords
-
-    def emit (self, record):
-        """Save message record. If storage site is exceeded, remove
-        oldest message record."""
-        if len(self.storage) >= self.maxrecords:
-            self.storage.pop()
-        self.storage.append(record)
-
-
 # all the URL classes
-from . import (fileurl, unknownurl, ftpurl, httpurl,
-    httpsurl, mailtourl, telneturl, nntpurl, ignoreurl)
+from . import (  # noqa: E402
+    fileurl,
+    unknownurl,
+    ftpurl,
+    httpurl,
+    dnsurl,
+    mailtourl,
+    telneturl,
+    nntpurl,
+    ignoreurl,
+    itmsservicesurl,
+)
