@@ -1,4 +1,3 @@
-# -*- coding: iso-8859-1 -*-
 # Copyright (C) 2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
@@ -27,14 +26,13 @@ class ItmsServicesUrl(urlbase.UrlBase):
 
     def check_syntax(self):
         """Only logs that this URL is unknown."""
-        super(ItmsServicesUrl, self).check_syntax()
-        if u"url=" not in self.urlparts[3]:
+        super().check_syntax()
+        if "url=" not in self.urlparts[3]:
             self.set_result(_("Missing required url parameter"), valid=False)
 
     def local_check(self):
         """Disable content checks."""
-        log.debug(LOG_CHECK, "Checking %s", unicode(self))
-        pass
+        log.debug(LOG_CHECK, "Checking %s", self)
 
     def check_content(self):
         """Allow recursion to check the url CGI param."""
