@@ -1,4 +1,3 @@
-# -*- coding: iso-8859-1 -*-
 # Copyright (C) 2000-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
@@ -19,67 +18,69 @@ Base URL handler.
 """
 import sys
 import os
-try:
-    import urlparse
-except ImportError:
-    # Python 3
-    from urllib import parse as urlparse
-try:
-    from urllib2 import urlopen
-except ImportError:
-    # Python 3
-    from urllib.request import urlopen
-import urllib
+import urllib.parse
+from urllib.request import urlopen
 import time
 import errno
 import socket
 import select
-try:
-    from cStringIO import StringIO
-except ImportError:
-    # Python 3
-    from io import StringIO
+from io import BytesIO
 
 from . import absolute_url, get_url_from
-from .. import (log, LOG_CHECK,
-  strformat, LinkCheckerError, url as urlutil, trace, get_link_pat)
+from .. import (
+    log,
+    LOG_CHECK,
+    strformat,
+    LinkCheckerError,
+    url as urlutil,
+    trace,
+    get_link_pat,
+)
+from ..htmlutil import htmlsoup
 from ..network import iputil
-from .const import (WARN_URL_EFFECTIVE_URL,
-    WARN_URL_ERROR_GETTING_CONTENT, WARN_URL_OBFUSCATED_IP,
-    WARN_URL_CONTENT_SIZE_ZERO, WARN_URL_CONTENT_SIZE_TOO_LARGE,
-    WARN_URL_WHITESPACE, URL_MAX_LENGTH, WARN_URL_TOO_LONG,
-    ExcList, ExcSyntaxList, ExcNoCacheList)
+from .const import (
+    WARN_URL_EFFECTIVE_URL,
+    WARN_URL_ERROR_GETTING_CONTENT,
+    WARN_URL_OBFUSCATED_IP,
+    WARN_URL_CONTENT_SIZE_ZERO,
+    WARN_URL_CONTENT_SIZE_TOO_LARGE,
+    WARN_URL_WHITESPACE,
+    URL_MAX_LENGTH,
+    WARN_URL_TOO_LONG,
+    ExcList,
+    ExcSyntaxList,
+    ExcNoCacheList,
+)
 from ..url import url_fix_wayback_query
 
-# helper alias
-unicode_safe = strformat.unicode_safe
-
 # schemes that are invalid with an empty hostname
 scheme_requires_host = ("ftp", "http", "telnet")
 
-def urljoin (parent, url):
+
+def urljoin(parent, url):
     """
     If url is relative, join parent and url. Else leave url as-is.
 
-    @return joined url
+    @return: joined url
     """
     if urlutil.url_is_absolute(url):
         return url
-    return urlparse.urljoin(parent, url)
+    return urllib.parse.urljoin(parent, url)
 
 
-def url_norm (url, encoding=None):
+def url_norm(url, encoding):
     """Wrapper for url.url_norm() to convert UnicodeError in
     LinkCheckerError."""
     try:
         return urlutil.url_norm(url, encoding=encoding)
     except UnicodeError:
-        msg = _("URL has unparsable domain name: %(name)s") % \
-            {"name": sys.exc_info()[1]}
+        msg = _("URL has unparsable domain name: %(name)s") % {
+            "name": sys.exc_info()[1]
+        }
         raise LinkCheckerError(msg)
 
 
-class UrlBase (object):
+class UrlBase:
     """An URL with additional information like validity etc."""
 
     # file types that can be parsed recursively
@@ -105,11 +106,22 @@ class UrlBase (object):
     }
 
     # Read in 16kb chunks
-    ReadChunkBytes = 1024*16
+    ReadChunkBytes = 1024 * 16
 
-    def __init__ (self, base_url, recursion_level, aggregate,
-                  parent_url=None, base_ref=None, line=-1, column=-1, page=-1,
-                  name=u"", url_encoding=None, extern=None):
+    def __init__(
+        self,
+        base_url,
+        recursion_level,
+        aggregate,
+        parent_url=None,
+        base_ref=None,
+        line=-1,
+        column=-1,
+        page=-1,
+        name="",
+        url_encoding=None,
+        extern=None,
+    ):
         """
         Initialize check data, and store given variables.
 
@@ -126,56 +138,84 @@ class UrlBase (object):
         @param extern: None or (is_extern, is_strict)
         """
         self.reset()
-        self.init(base_ref, base_url, parent_url, recursion_level,
-                  aggregate, line, column, page, name, url_encoding, extern)
+        self.init(
+            base_ref,
+            base_url,
+            parent_url,
+            recursion_level,
+            aggregate,
+            line,
+            column,
+            page,
+            name,
+            url_encoding,
+            extern,
+        )
         self.check_syntax()
         if recursion_level == 0:
             self.add_intern_pattern()
         self.set_extern(self.url)
         if self.extern[0] and self.extern[1]:
-            self.add_info(_("The URL is outside of the domain "
-                          "filter, checked only syntax."))
+            self.add_info(
+                _("The URL is outside of the domain filter, checked only syntax.")
+            )
             if not self.has_result:
                 self.set_result(_("filtered"))
 
-    def init (self, base_ref, base_url, parent_url, recursion_level,
-              aggregate, line, column, page, name, url_encoding, extern):
+    def init(
+        self,
+        base_ref,
+        base_url,
+        parent_url,
+        recursion_level,
+        aggregate,
+        line,
+        column,
+        page,
+        name,
+        url_encoding,
+        extern,
+    ):
         """
         Initialize internal data.
         """
         self.base_ref = base_ref
         if self.base_ref is not None:
-            assert isinstance(self.base_ref, unicode), repr(self.base_ref)
+            assert isinstance(self.base_ref, str), repr(self.base_ref)
         self.base_url = base_url.strip() if base_url else base_url
         if self.base_url is not None:
-            assert isinstance(self.base_url, unicode), repr(self.base_url)
+            assert isinstance(self.base_url, str), repr(self.base_url)
         self.parent_url = parent_url
         if self.parent_url is not None:
-            assert isinstance(self.parent_url, unicode), repr(self.parent_url)
+            assert isinstance(self.parent_url, str), repr(self.parent_url)
         self.recursion_level = recursion_level
         self.aggregate = aggregate
         self.line = line
         self.column = column
         self.page = page
         self.name = name
-        assert isinstance(self.name, unicode), repr(self.name)
+        assert isinstance(self.name, str), repr(self.name)
         self.encoding = url_encoding
-        self.charset = None
         self.extern = extern
         if self.base_ref:
-            assert not urlutil.url_needs_quoting(self.base_ref), \
-                   "unquoted base reference URL %r" % self.base_ref
+            assert not urlutil.url_needs_quoting(self.base_ref), (
+                "unquoted base reference URL %r" % self.base_ref
+            )
         if self.parent_url:
-            assert not urlutil.url_needs_quoting(self.parent_url), \
-                   "unquoted parent URL %r" % self.parent_url
+            assert not urlutil.url_needs_quoting(self.parent_url), (
+                "unquoted parent URL %r" % self.parent_url
+            )
         url = absolute_url(self.base_url, base_ref, parent_url)
         # assume file link if no scheme is found
         self.scheme = url.split(":", 1)[0].lower() or "file"
         if self.base_url != base_url:
-            self.add_warning(_("Leading or trailing whitespace in URL `%(url)s'.") %
-                               {"url": base_url}, tag=WARN_URL_WHITESPACE)
+            self.add_warning(
+                _("Leading or trailing whitespace in URL `%(url)s'.")
+                % {"url": base_url},
+                tag=WARN_URL_WHITESPACE,
+            )
 
-    def reset (self):
+    def reset(self):
         """
         Reset all variables to default values.
         """
@@ -189,7 +229,7 @@ class UrlBase (object):
         # the scheme, host, port and anchor part of url
         self.scheme = self.host = self.port = self.anchor = None
         # the result message string and flag
-        self.result = u""
+        self.result = ""
         self.has_result = False
         # valid or not
         self.valid = True
@@ -199,7 +239,8 @@ class UrlBase (object):
         self.info = []
         # content size
         self.size = -1
-        # last modification time of content in HTTP-date format as specified in RFC2616 chapter 3.3.1
+        # last modification time of content in HTTP-date format
+        # as specified in RFC2616 chapter 3.3.1
         self.modified = None
         # download time
         self.dltime = -1
@@ -209,6 +250,10 @@ class UrlBase (object):
         self.url_connection = None
         # data of url content,  (data == None) means no data is available
         self.data = None
+        # url content as a Unicode string
+        self.text = None
+        # url content as a Beautiful Soup object
+        self.soup = None
         # cache url is set by build_url() calling set_cache_url()
         self.cache_url = None
         # extern flags (is_extern, is_strict)
@@ -220,33 +265,36 @@ class UrlBase (object):
         # flag if content should be checked or not
         self.do_check_content = True
         # MIME content type
-        self.content_type = u""
+        self.content_type = ""
         # URLs seen through redirections
         self.aliases = []
 
-    def set_result (self, msg, valid=True, overwrite=False):
+    def set_result(self, msg, valid=True, overwrite=False):
         """
         Set result string and validity.
         """
         if self.has_result and not overwrite:
-            log.warn(LOG_CHECK,
-              "Double result %r (previous %r) for %s", msg, self.result, self)
+            log.warn(
+                LOG_CHECK,
+                "Double result %r (previous %r) for %s",
+                msg,
+                self.result,
+                self,
+            )
         else:
             self.has_result = True
-        if not isinstance(msg, unicode):
-            log.warn(LOG_CHECK, "Non-unicode result for %s: %r", self, msg)
-        elif not msg:
+        if not msg:
             log.warn(LOG_CHECK, "Empty result for %s", self)
         self.result = msg
         self.valid = valid
         # free content data
         self.data = None
 
-    def get_title (self):
+    def get_title(self):
         """Return title of page the URL refers to.
         This is per default the filename or the URL."""
         if self.title is None:
-            url = u""
+            url = ""
             if self.base_url:
                 url = self.base_url
             elif self.url:
@@ -258,17 +306,17 @@ class UrlBase (object):
                     self.title = title
         return self.title
 
-    def is_parseable (self):
+    def is_parseable(self):
         """
         Return True iff content of this url is parseable.
         """
         return False
 
-    def is_html (self):
+    def is_html(self):
         """Return True iff content of this url is HTML formatted."""
         return self._is_ctype("html")
 
-    def is_css (self):
+    def is_css(self):
         """Return True iff content of this url is CSS stylesheet."""
         return self._is_ctype("css")
 
@@ -279,11 +327,11 @@ class UrlBase (object):
         mime = self.content_type
         return self.ContentMimetypes.get(mime) == ctype
 
-    def is_http (self):
+    def is_http(self):
         """Return True for http:// or https:// URLs."""
         return self.scheme in ("http", "https")
 
-    def is_file (self):
+    def is_file(self):
         """Return True for file:// URLs."""
         return self.scheme == "file"
 
@@ -295,31 +343,35 @@ class UrlBase (object):
         """Return True for local (ie. file://) URLs."""
         return self.is_file()
 
-    def add_warning (self, s, tag=None):
+    def add_warning(self, s, tag=None):
         """
         Add a warning string.
         """
         item = (tag, s)
-        if item not in self.warnings and \
-           tag not in self.aggregate.config["ignorewarnings"]:
+        if (
+            item not in self.warnings
+            and tag not in self.aggregate.config["ignorewarnings"]
+        ):
             self.warnings.append(item)
 
-    def add_info (self, s):
+    def add_info(self, s):
         """
         Add an info string.
         """
         if s not in self.info:
             self.info.append(s)
 
-    def set_cache_url (self):
+    def set_cache_url(self):
         """Set the URL to be used for caching."""
-        # remove anchor from cached target url since we assume
-        # URLs with different anchors to have the same content
-        self.cache_url = urlutil.urlunsplit(self.urlparts[:4]+[u''])
-        if self.cache_url is not None:
-            assert isinstance(self.cache_url, unicode), repr(self.cache_url)
+        if "AnchorCheck" in self.aggregate.config["enabledplugins"]:
+            self.cache_url = self.url
+        else:
+            # remove anchor from cached target url since we assume
+            # URLs with different anchors to have the same content
+            self.cache_url = urlutil.urlunsplit(self.urlparts[:4] + [''])
+        log.debug(LOG_CHECK, "cache_url '%s'", self.cache_url)
 
-    def check_syntax (self):
+    def check_syntax(self):
         """
         Called before self.check(), this function inspects the
         url syntax. Success enables further checking, failure
@@ -328,7 +380,7 @@ class UrlBase (object):
         """
         log.debug(LOG_CHECK, "checking syntax")
         if self.base_url is None:
-            self.base_url = u""
+            self.base_url = ""
         if not (self.base_url or self.parent_url):
             self.set_result(_("URL is empty"), valid=False)
             return
@@ -336,7 +388,7 @@ class UrlBase (object):
             self.build_url()
             self.check_url_warnings()
         except tuple(ExcSyntaxList) as msg:
-            self.set_result(unicode_safe(msg), valid=False)
+            self.set_result(str(msg), valid=False)
         else:
             self.set_cache_url()
 
@@ -344,15 +396,19 @@ class UrlBase (object):
         """Check URL name and length."""
         effectiveurl = urlutil.urlunsplit(self.urlparts)
         if self.url != effectiveurl:
-            self.add_warning(_("Effective URL %(url)r.") %
-                             {"url": effectiveurl},
-                             tag=WARN_URL_EFFECTIVE_URL)
+            self.add_warning(
+                _("Effective URL %(url)r.") % {"url": effectiveurl},
+                tag=WARN_URL_EFFECTIVE_URL,
+            )
             self.url = effectiveurl
-        if len(self.url) > URL_MAX_LENGTH and self.scheme != u"data":
+        if len(self.url) > URL_MAX_LENGTH and self.scheme != "data":
             args = dict(len=len(self.url), max=URL_MAX_LENGTH)
-            self.add_warning(_("URL length %(len)d is longer than %(max)d.") % args, tag=WARN_URL_TOO_LONG)
+            self.add_warning(
+                _("URL length %(len)d is longer than %(max)d.") % args,
+                tag=WARN_URL_TOO_LONG,
+            )
 
-    def build_url (self):
+    def build_url(self):
         """
         Construct self.url and self.urlparts out of the given base
         url information self.base_url, self.parent_url and self.base_ref.
@@ -368,39 +424,47 @@ class UrlBase (object):
             self.url = urljoin(self.base_ref, base_url)
         elif self.parent_url:
             # strip the parent url query and anchor
-            urlparts = list(urlparse.urlsplit(self.parent_url))
+            urlparts = list(urllib.parse.urlsplit(self.parent_url))
             urlparts[4] = ""
             parent_url = urlutil.urlunsplit(urlparts)
             self.url = urljoin(parent_url, base_url)
         else:
             self.url = base_url
         # urljoin can unnorm the url path, so norm it again
-        urlparts = list(urlparse.urlsplit(self.url))
+        urlparts = list(urllib.parse.urlsplit(self.url))
         if urlparts[2]:
             urlparts[2] = urlutil.collapse_segments(urlparts[2])
             if not urlparts[0].startswith("feed"):
-                urlparts[2] = url_fix_wayback_query(urlparts[2]) # restore second / in http[s]:// in wayback path
+                # restore second / in http[s]:// in wayback path
+                urlparts[2] = url_fix_wayback_query(urlparts[2])
         self.url = urlutil.urlunsplit(urlparts)
-        # split into (modifiable) list
-        self.urlparts = strformat.url_unicode_split(self.url)
-        self.build_url_parts()
+        self.urlparts = self.build_url_parts(self.url)
         # and unsplit again
         self.url = urlutil.urlunsplit(self.urlparts)
 
-    def build_url_parts (self):
-        """Set userinfo, host, port and anchor from self.urlparts.
+    def build_url_parts(self, url):
+        """Set userinfo, host, port and anchor from url and return urlparts.
         Also checks for obfuscated IP addresses.
         """
+        split = urllib.parse.urlsplit(url)
+        urlparts = list(split)
         # check userinfo@host:port syntax
-        self.userinfo, host = urllib.splituser(self.urlparts[1])
-        port = urlutil.default_ports.get(self.scheme, 0)
-        host, port = urlutil.splitport(host, port=port)
+        self.userinfo, host = urlutil.split_netloc(split.netloc)
+        try:
+            port = split.port
+        except ValueError:
+            raise LinkCheckerError(
+                _("URL host %(host)r has invalid port") % {"host": host}
+            )
         if port is None:
-            raise LinkCheckerError(_("URL host %(host)r has invalid port") %
-                    {"host": host})
+            port = urlutil.default_ports.get(self.scheme, 0)
+        if port is None:
+            raise LinkCheckerError(
+                _("URL host %(host)r has invalid port") % {"host": host}
+            )
         self.port = port
-        # set host lowercase
-        self.host = host.lower()
+        # urllib.parse.SplitResult.hostname is lowercase
+        self.host = split.hostname
         if self.scheme in scheme_requires_host:
             if not self.host:
                 raise LinkCheckerError(_("URL has empty hostname"))
@@ -410,15 +474,16 @@ class UrlBase (object):
         else:
             host = "%s:%d" % (self.host, self.port)
         if self.userinfo:
-            self.urlparts[1] = "%s@%s" % (self.userinfo, host)
+            urlparts[1] = "%s@%s" % (self.userinfo, host)
         else:
-            self.urlparts[1] = host
+            urlparts[1] = host
         # safe anchor for later checking
-        self.anchor = self.urlparts[4]
+        self.anchor = split.fragment
         if self.anchor is not None:
-            assert isinstance(self.anchor, unicode), repr(self.anchor)
+            assert isinstance(self.anchor, str), repr(self.anchor)
+        return urlparts
 
-    def check_obfuscated_ip (self):
+    def check_obfuscated_ip(self):
         """Warn if host of this URL is obfuscated IP address."""
         # check if self.host can be an IP address
         # check for obfuscated IP address
@@ -427,11 +492,12 @@ class UrlBase (object):
             if ips:
                 self.host = ips[0]
                 self.add_warning(
-                   _("URL %(url)s has obfuscated IP address %(ip)s") % \
-                   {"url": self.base_url, "ip": ips[0]},
-                          tag=WARN_URL_OBFUSCATED_IP)
+                    _("URL %(url)s has obfuscated IP address %(ip)s")
+                    % {"url": self.base_url, "ip": ips[0]},
+                    tag=WARN_URL_OBFUSCATED_IP,
+                )
 
-    def check (self):
+    def check(self):
         """Main check function for checking this URL."""
         if self.aggregate.config["trace"]:
             trace.trace_on()
@@ -446,9 +512,9 @@ class UrlBase (object):
             else:
                 raise
 
-    def local_check (self):
+    def local_check(self):
         """Local check function can be overridden in subclasses."""
-        log.debug(LOG_CHECK, "Checking %s", unicode(self))
+        log.debug(LOG_CHECK, "Checking %s", self)
         # strict extern URLs should not be checked
         assert not self.extern[1], 'checking strict extern URL'
         # check connection
@@ -465,8 +531,11 @@ class UrlBase (object):
                 value = _('Hostname not found')
             elif isinstance(exc, UnicodeError):
                 # idna.encode(host) failed
-                value = _('Bad hostname %(host)r: %(msg)s') % {'host': self.host, 'msg': str(value)}
-            self.set_result(unicode_safe(value), valid=False)
+                value = _('Bad hostname %(host)r: %(msg)s') % {
+                    'host': self.host,
+                    'msg': value,
+                }
+            self.set_result(value, valid=False)
 
     def check_content(self):
         """Check content of URL.
@@ -481,11 +550,13 @@ class UrlBase (object):
                     return True
             except tuple(ExcList):
                 value = self.handle_exception()
-                self.add_warning(_("could not get content: %(msg)s") %
-                     {"msg": str(value)}, tag=WARN_URL_ERROR_GETTING_CONTENT)
+                self.add_warning(
+                    _("could not get content: %(msg)s") % {"msg": value},
+                    tag=WARN_URL_ERROR_GETTING_CONTENT,
+                )
         return False
 
-    def close_connection (self):
+    def close_connection(self):
         """
         Close an opened url connection.
         """
@@ -499,43 +570,49 @@ class UrlBase (object):
             pass
         self.url_connection = None
 
-    def handle_exception (self):
+    def handle_exception(self):
         """
         An exception occurred. Log it and set the cache flag.
         """
         etype, evalue = sys.exc_info()[:2]
-        log.debug(LOG_CHECK, "Error in %s: %s %s", self.url, etype, evalue, exception=True)
+        log.debug(
+            LOG_CHECK, "Error in %s: %s %s", self.url, etype, evalue, exception=True
+        )
         # note: etype must be the exact class, not a subclass
-        if (etype in ExcNoCacheList) or \
-           (etype == socket.error and evalue.args[0]==errno.EBADF) or \
-            not evalue:
+        if (
+            (etype in ExcNoCacheList)
+            or (etype == socket.error and evalue.args[0] == errno.EBADF)
+            or not evalue
+        ):
             # EBADF occurs when operating on an already socket
             self.caching = False
-        # format unicode message "<exception name>: <error message>"
-        errmsg = unicode(etype.__name__)
-        uvalue = strformat.unicode_safe(evalue)
-        if uvalue:
-            errmsg += u": %s" % uvalue
+        # format message "<exception name>: <error message>"
+        errmsg = etype.__name__
+        if evalue:
+            errmsg += ": %s" % evalue
         # limit length to 240
         return strformat.limit(errmsg, length=240)
 
-    def check_connection (self):
+    def check_connection(self):
         """
         The basic connection check uses urlopen to initialize
         a connection object.
         """
         self.url_connection = urlopen(self.url)
 
-    def add_size_info (self):
+    def add_size_info(self):
         """Set size of URL content (if any)..
         Should be overridden in subclasses."""
         maxbytes = self.aggregate.config["maxfilesizedownload"]
         if self.size > maxbytes:
             self.add_warning(
-              _("Content size %(size)s is larger than %(maxbytes)s.") %
-                  dict(size=strformat.strsize(self.size),
-                       maxbytes=strformat.strsize(maxbytes)),
-                tag=WARN_URL_CONTENT_SIZE_TOO_LARGE)
+                _("Content size %(size)s is larger than %(maxbytes)s.")
+                % dict(
+                    size=strformat.strsize(self.size),
+                    maxbytes=strformat.strsize(maxbytes),
+                ),
+                tag=WARN_URL_CONTENT_SIZE_TOO_LARGE,
+            )
 
     def allows_simple_recursion(self):
         """Check recursion level and extern status."""
@@ -548,7 +625,7 @@ class UrlBase (object):
             return False
         return True
 
-    def allows_recursion (self):
+    def allows_recursion(self):
         """
         Return True iff we can recurse into the url's content.
         """
@@ -577,7 +654,7 @@ class UrlBase (object):
         """Returns True: only check robots.txt on HTTP links."""
         return True
 
-    def set_extern (self, url):
+    def set_extern(self, url):
         """
         Match URL against extern and intern link patterns. If no pattern
         matches the URL is extern. Sets self.extern to a tuple (bool,
@@ -592,15 +669,13 @@ class UrlBase (object):
             return
         for entry in self.aggregate.config["externlinks"]:
             match = entry['pattern'].search(url)
-            if (entry['negate'] and not match) or \
-               (match and not entry['negate']):
+            if (entry['negate'] and not match) or (match and not entry['negate']):
                 log.debug(LOG_CHECK, "Extern URL %r", url)
                 self.extern = (1, entry['strict'])
                 return
         for entry in self.aggregate.config["internlinks"]:
             match = entry['pattern'].search(url)
-            if (entry['negate'] and not match) or \
-               (match and not entry['negate']):
+            if (entry['negate'] and not match) or (match and not entry['negate']):
                 log.debug(LOG_CHECK, "Intern URL %r", url)
                 self.extern = (0, 0)
                 return
@@ -609,33 +684,56 @@ class UrlBase (object):
         else:
             self.extern = (1, 1)
 
-    def set_content_type (self):
+    def set_content_type(self):
         """Set content MIME type.
         Should be overridden in subclasses."""
         pass
 
-    def can_get_content (self):
+    def can_get_content(self):
         """Indicate wether url get_content() can be called."""
         return self.size <= self.aggregate.config["maxfilesizedownload"]
 
-    def get_content (self):
-        """Precondition: url_connection is an opened URL."""
+    def download_content(self):
+        log.debug(LOG_CHECK, "Get content of %r", self.url)
+        t = time.time()
+        content = self.read_content()
+        self.size = len(content)
+        self.dltime = time.time() - t
+        if self.size == 0:
+            self.add_warning(_("Content size is zero."), tag=WARN_URL_CONTENT_SIZE_ZERO)
+        else:
+            self.aggregate.add_downloaded_bytes(self.size)
+        return content
+
+    def get_soup(self):
+        if self.soup is None:
+            self.get_content()
+        return self.soup
+
+    def get_raw_content(self):
         if self.data is None:
-            log.debug(LOG_CHECK, "Get content of %r", self.url)
-            t = time.time()
-            self.data = self.read_content()
-            self.size = len(self.data)
-            self.dltime = time.time() - t
-            if self.size == 0:
-                self.add_warning(_("Content size is zero."),
-                             tag=WARN_URL_CONTENT_SIZE_ZERO)
-            else:
-                self.aggregate.add_downloaded_bytes(self.size)
+            self.data = self.download_content()
         return self.data
 
+    def get_content(self, encoding=None):
+        if self.text is None:
+            self.get_raw_content()
+            self.soup = htmlsoup.make_soup(self.data, encoding)
+            # Sometimes soup.original_encoding is None!  Better mangled text
+            # than an internal crash, eh?  ISO-8859-1 is a safe fallback in the
+            # sense that any binary blob can be decoded, it'll never cause a
+            # UnicodeDecodeError.
+            log.debug(
+                LOG_CHECK, "Beautiful Soup detected %s", self.soup.original_encoding
+            )
+            self.encoding = self.soup.original_encoding or 'ISO-8859-1'
+            log.debug(LOG_CHECK, "Content encoding %s", self.encoding)
+            self.text = self.data.decode(self.encoding)
+        return self.text
+
     def read_content(self):
         """Return data for this URL. Can be overridden in subclasses."""
-        buf = StringIO()
+        buf = BytesIO()
         data = self.read_content_chunk()
         while data:
             if buf.tell() + len(data) > self.aggregate.config["maxfilesizedownload"]:
@@ -645,55 +743,70 @@ class UrlBase (object):
         return buf.getvalue()
 
     def read_content_chunk(self):
-        """Read one chunk of content from this URL."""
+        """Read one chunk of content from this URL.
+        Precondition: url_connection is an opened URL.
+        """
         return self.url_connection.read(self.ReadChunkBytes)
 
-    def get_user_password (self):
+    def get_user_password(self):
         """Get tuple (user, password) from configured authentication.
         Both user and password can be None.
         """
         if self.userinfo:
             # URL itself has authentication info
-            return urllib.splitpasswd(self.userinfo)
+            split = urllib.parse.urlsplit(self.url)
+            return (split.username, split.password)
         return self.aggregate.config.get_user_password(self.url)
 
-    def add_url (self, url, line=0, column=0, page=0, name=u"", base=None):
+    def add_url(self, url, line=0, column=0, page=0, name="", base=None):
         """Add new URL to queue."""
         if base:
-            base_ref = urlutil.url_norm(base)[0]
+            base_ref = urlutil.url_norm(base, encoding=self.encoding)[0]
         else:
             base_ref = None
-        url_data = get_url_from(url, self.recursion_level+1, self.aggregate,
-            parent_url=self.url, base_ref=base_ref, line=line, column=column,
-            page=page, name=name, parent_content_type=self.content_type)
+        url_data = get_url_from(
+            url,
+            self.recursion_level + 1,
+            self.aggregate,
+            parent_url=self.url,
+            base_ref=base_ref,
+            line=line,
+            column=column,
+            page=page,
+            name=name,
+            parent_content_type=self.content_type,
+            url_encoding=self.encoding,
+        )
         self.aggregate.urlqueue.put(url_data)
 
-    def serialized (self, sep=os.linesep):
+    def serialized(self, sep=os.linesep):
         """
         Return serialized url check data as unicode string.
         """
-        return unicode_safe(sep).join([
-            u"%s link" % self.scheme,
-            u"base_url=%r" % self.base_url,
-            u"parent_url=%r" % self.parent_url,
-            u"base_ref=%r" % self.base_ref,
-            u"recursion_level=%d" % self.recursion_level,
-            u"url_connection=%s" % self.url_connection,
-            u"line=%d" % self.line,
-            u"column=%d" % self.column,
-            u"page=%d" % self.page,
-            u"name=%r" % self.name,
-            u"anchor=%r" % self.anchor,
-            u"cache_url=%s" % self.cache_url,
-           ])
+        return sep.join(
+            [
+                "%s link" % self.scheme,
+                "base_url=%r" % self.base_url,
+                "parent_url=%r" % self.parent_url,
+                "base_ref=%r" % self.base_ref,
+                "recursion_level=%d" % self.recursion_level,
+                "url_connection=%s" % self.url_connection,
+                "line=%s" % self.line,
+                "column=%s" % self.column,
+                "page=%d" % self.page,
+                "name=%r" % self.name,
+                "anchor=%r" % self.anchor,
+                "cache_url=%s" % self.cache_url,
+            ]
+        )
 
-    def get_intern_pattern (self, url=None):
+    def get_intern_pattern(self, url=None):
         """Get pattern for intern URL matching.
 
         @param url: the URL to set intern pattern for, else self.url
-        @ptype url: unicode or None
-        @return non-empty regex pattern or None
-        @rtype String or None
+        @type url: unicode or None
+        @return: non-empty regex pattern or None
+        @rtype: String or None
         """
         return None
 
@@ -705,11 +818,10 @@ class UrlBase (object):
                 log.debug(LOG_CHECK, "Add intern pattern %r", pat)
                 self.aggregate.config['internlinks'].append(get_link_pat(pat))
         except UnicodeError as msg:
-            res = _("URL has unparsable domain name: %(domain)s") % \
-                {"domain": msg}
+            res = _("URL has unparsable domain name: %(domain)s") % {"domain": msg}
             self.set_result(res, valid=False)
 
-    def __unicode__(self):
+    def __str__(self):
         """
         Get URL info.
 
@@ -718,29 +830,30 @@ class UrlBase (object):
         """
         return self.serialized()
 
-    def __str__(self):
+    def __bytes__(self):
         """
         Get URL info.
 
         @return: URL info, encoded with the output logger encoding
         @rtype: string
         """
-        s = unicode(self)
+        s = str(self)
         return self.aggregate.config['logger'].encode(s)
 
-    def __repr__ (self):
+    def __repr__(self):
         """
         Get URL info.
 
         @return: URL info
         @rtype: unicode
         """
-        return u"<%s>" % self.serialized(sep=u", ")
+        return "<%s>" % self.serialized(sep=", ")
 
-    def to_wire_dict (self):
+    def to_wire_dict(self):
         """Return a simplified transport object for logging and caching.
 
         The transport object must contain these attributes:
+
         - url_data.valid: bool
           Indicates if URL is valid
         - url_data.result: unicode
@@ -766,9 +879,9 @@ class UrlBase (object):
         - url_data.info: list of unicode
           Additional information about this URL.
         - url_data.line: int
-          Line number of this URL at parent document, or -1
+          Line number of this URL at parent document, or None
         - url_data.column: int
-          Column number of this URL at parent document, or -1
+          Column number of this URL at parent document, or None
         - url_data.page: int
           Page number of this URL at parent document, or -1
         - url_data.cache_url: unicode
@@ -780,31 +893,32 @@ class UrlBase (object):
         - url_data.last_modified: datetime
           Last modification date of retrieved page (or None).
         """
-        return dict(valid=self.valid,
-          extern=self.extern[0],
-          result=self.result,
-          warnings=self.warnings[:],
-          name=self.name or u"",
-          title=self.get_title(),
-          parent_url=self.parent_url or u"",
-          base_ref=self.base_ref or u"",
-          base_url=self.base_url or u"",
-          url=self.url or u"",
-          domain=(self.urlparts[1] if self.urlparts else u""),
-          checktime=self.checktime,
-          dltime=self.dltime,
-          size=self.size,
-          info=self.info,
-          line=self.line,
-          column=self.column,
-          page=self.page,
-          cache_url=self.cache_url,
-          content_type=self.content_type,
-          level=self.recursion_level,
-          modified=self.modified,
+        return dict(
+            valid=self.valid,
+            extern=self.extern[0],
+            result=self.result,
+            warnings=self.warnings[:],
+            name=self.name or "",
+            title=self.get_title(),
+            parent_url=self.parent_url or "",
+            base_ref=self.base_ref or "",
+            base_url=self.base_url or "",
+            url=self.url or "",
+            domain=(self.urlparts[1] if self.urlparts else ""),
+            checktime=self.checktime,
+            dltime=self.dltime,
+            size=self.size,
+            info=self.info,
+            line=self.line,
+            column=self.column,
+            page=self.page,
+            cache_url=self.cache_url,
+            content_type=self.content_type,
+            level=self.recursion_level,
+            modified=self.modified,
         )
 
-    def to_wire (self):
+    def to_wire(self):
         """Return compact UrlData object with information from to_wire_dict().
         """
         return CompactUrlData(self.to_wire_dict())
@@ -835,8 +949,10 @@ urlDataAttr = [
     'level',
 ]
 
-class CompactUrlData (object):
+
+class CompactUrlData:
     """Store selected UrlData attributes in slots to minimize memory usage."""
+
     __slots__ = urlDataAttr
 
     def __init__(self, wired_url_data):
