@@ -1,5 +1,4 @@
-# -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2011 Bastian Kleineidam
+# Copyright (C) 2000-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +17,6 @@ import re
 import linecache
 import time
 import sys
-import thread
 import threading
 
 # tracing
@@ -26,7 +24,7 @@ _trace_ignore = set()
 _trace_filter = set()
 
 
-def trace_ignore (names):
+def trace_ignore(names):
     """Add given names to trace ignore set, or clear set if names is None."""
     if names is None:
         _trace_ignore.clear()
@@ -34,7 +32,7 @@ def trace_ignore (names):
         _trace_ignore.update(names)
 
 
-def trace_filter (patterns):
+def trace_filter(patterns):
     """Add given patterns to trace filter set or clear set if patterns is
     None."""
     if patterns is None:
@@ -43,19 +41,19 @@ def trace_filter (patterns):
         _trace_filter.update(re.compile(pat) for pat in patterns)
 
 
-def _trace (frame, event, arg):
+def _trace(frame, event, arg):
     """Trace function calls."""
     if event in ('call', 'c_call'):
         _trace_line(frame, event, arg)
     elif event in ('return', 'c_return'):
         _trace_line(frame, event, arg)
-        print "  return:", arg
-    #elif event in ('exception', 'c_exception'):
+        print("  return:", arg)
+    # elif event in ('exception', 'c_exception'):
     #    _trace_line(frame, event, arg)
     return _trace
 
 
-def _trace_full (frame, event, arg):
+def _trace_full(frame, event, arg):
     """Trace every executed line."""
     if event == "line":
         _trace_line(frame, event, arg)
@@ -64,7 +62,7 @@ def _trace_full (frame, event, arg):
     return _trace_full
 
 
-def _trace_line (frame, event, arg):
+def _trace_line(frame, event, arg):
     """Print current executed line."""
     name = frame.f_globals["__name__"]
     if name in _trace_ignore:
@@ -77,13 +75,14 @@ def _trace_line (frame, event, arg):
     if filename.endswith((".pyc", ".pyo")):
         filename = filename[:-1]
     line = linecache.getline(filename, lineno)
-    tid = thread.get_ident()
-    tname = threading.currentThread().getName()
+    currentThread = threading.currentThread()
+    tid = currentThread.ident
+    tname = currentThread.getName()
     args = (tid, tname, time.time(), line.rstrip(), name, lineno)
-    print "THREAD(%d) %r %.2f %s # %s:%d" % args
+    print("THREAD(%d) %r %.2f %s # %s:%d" % args)
 
 
-def trace_on (full=False):
+def trace_on(full=False):
     """Start tracing of the current thread (and the current thread only)."""
     if full:
         sys.settrace(_trace_full)
@@ -91,6 +90,6 @@ def trace_on (full=False):
         sys.settrace(_trace)
 
 
-def trace_off ():
+def trace_off():
     """Stop tracing of the current thread (and the current thread only)."""
     sys.settrace(None)
