@@ -1,4 +1,3 @@
-# -*- coding: iso-8859-1 -*-
 # Copyright (C) 2000-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
@@ -20,20 +19,11 @@ Functions for parsing and matching URL strings.
 
 import os
 import re
-try:  # Python 3
-    from urllib import parse
-    from urllib import parse as urlparse
-except ImportError:  # Python 2
-    import urllib as parse
-    import urlparse
-
-import requests
-
-from . import log, LOG_CHECK
+import urllib.parse
 
 for scheme in ('ldap', 'irc'):
-    if scheme not in urlparse.uses_netloc:
-        urlparse.uses_netloc.append(scheme)
+    if scheme not in urllib.parse.uses_netloc:
+        urllib.parse.uses_netloc.append(scheme)
 
 # The character set to encode non-ASCII characters in a URL. See also
 # http://tools.ietf.org/html/rfc2396#section-2.1
@@ -43,13 +33,6 @@ for scheme in ('ldap', 'irc'):
 # http://code.google.com/p/browsersec/wiki/Part1#Unicode_in_URLs
 url_encoding = "utf-8"
 
-
-# constants defining url part indexes
-SCHEME = 0
-HOSTNAME = DOMAIN = 1
-PORT = 2
-DOCUMENT = 3
-
 default_ports = {
     'http': 80,
     'https': 443,
@@ -66,33 +49,34 @@ _basic = {
     "_hex_full": r"0-9a-f",
     "_part": r"([a-z0-9][-a-z0-9]{0,61}|[a-z])",
 }
-_safe_char = r"([a-z0-9%(_path)s\+]|"\
-             r"(%%[%(_hex_safe)s][%(_hex_full)s]))" % _basic
+_safe_char = (
+    r"([a-z0-9%(_path)s\+]|"
+    r"(%%[%(_hex_safe)s][%(_hex_full)s]))" % _basic
+)
 _safe_scheme_pattern = r"(https?|ftp)"
 _safe_domain_pattern = r"(%(_part)s(\.%(_part)s)*\.?)" % _basic
-_safe_host_pattern = _safe_domain_pattern+r"(:(80|8080|8000|443))?" % _basic
-_safe_path_pattern = r"((/([a-z0-9%(_path)s]|"\
-                     r"(%%[%(_hex_safe)s][%(_hex_full)s]))+)*/?)" % _basic
+_safe_host_pattern = _safe_domain_pattern + r"(:(80|8080|8000|443))?" % _basic
+_safe_path_pattern = (
+    r"((/([a-z0-9%(_path)s]|"
+    r"(%%[%(_hex_safe)s][%(_hex_full)s]))+)*/?)" % _basic
+)
 _safe_fragment_pattern = r"%s*" % _safe_char
 _safe_cgi = r"%s+(=(%s|/)+)?" % (_safe_char, _safe_char)
 _safe_query_pattern = r"(%s(&%s)*)?" % (_safe_cgi, _safe_cgi)
 _safe_param_pattern = r"(%s(;%s)*)?" % (_safe_cgi, _safe_cgi)
-safe_url_pattern = r"%s://%s%s(#%s)?" % \
-    (_safe_scheme_pattern, _safe_host_pattern,
-     _safe_path_pattern, _safe_fragment_pattern)
+safe_url_pattern = r"%s://%s%s(#%s)?" % (
+    _safe_scheme_pattern,
+    _safe_host_pattern,
+    _safe_path_pattern,
+    _safe_fragment_pattern,
+)
 
-is_safe_char = re.compile("(?i)^%s$" % _safe_char).match
 is_safe_url = re.compile("(?i)^%s$" % safe_url_pattern).match
 is_safe_domain = re.compile("(?i)^%s$" % _safe_domain_pattern).match
-is_safe_host = re.compile("(?i)^%s$" % _safe_host_pattern).match
-is_safe_path = re.compile("(?i)^%s$" % _safe_path_pattern).match
-is_safe_parameter = re.compile("(?i)^%s$" % _safe_param_pattern).match
-is_safe_query = re.compile("(?i)^%s$" % _safe_query_pattern).match
-is_safe_fragment = re.compile("(?i)^%s$" % _safe_fragment_pattern).match
 
 
 # snatched form urlparse.py
-def splitparams (path):
+def splitparams(path):
     """Split off parameter part from path.
     Returns tuple (path-without-param, param)
     """
@@ -102,10 +86,10 @@ def splitparams (path):
         i = path.find(';')
     if i < 0:
         return path, ''
-    return path[:i], path[i+1:]
+    return path[:i], path[i + 1:]
 
 
-def is_numeric_port (portstr):
+def is_numeric_port(portstr):
     """return: integer port (== True) iff portstr is a valid port number,
            False otherwise
     """
@@ -117,13 +101,7 @@ def is_numeric_port (portstr):
     return False
 
 
-def safe_host_pattern (host):
-    """Return regular expression pattern with given host for URL testing."""
-    return "(?i)%s://%s%s(#%s)?" % \
-     (_safe_scheme_pattern, host, _safe_path_pattern, _safe_fragment_pattern)
-
-
-def parse_qsl (qs, keep_blank_values=0, strict_parsing=0):
+def parse_qsl(qs, encoding, keep_blank_values=0, strict_parsing=0):
     """Parse a query given as a string argument.
 
     @param qs: URL-encoded query string to be parsed
@@ -164,21 +142,21 @@ def parse_qsl (qs, keep_blank_values=0, strict_parsing=0):
             else:
                 continue
         if nv[1] or keep_blank_values:
-            name = parse.unquote(nv[0].replace('+', ' '))
+            name = urllib.parse.unquote(nv[0].replace('+', ' '), encoding=encoding)
             if nv[1]:
-                value = parse.unquote(nv[1].replace('+', ' '))
+                value = urllib.parse.unquote(nv[1].replace('+', ' '), encoding=encoding)
             else:
                 value = nv[1]
             r.append((name, value, sep))
     return r
 
 
-def idna_encode (host):
+def idna_encode(host):
     """Encode hostname as internationalized domain name (IDN) according
     to RFC 3490.
     @raise: UnicodeError if hostname is not properly IDN encoded.
     """
-    if host and isinstance(host, unicode):
+    if host:
         try:
             host.encode('ascii')
             return host, False
@@ -188,26 +166,39 @@ def idna_encode (host):
     return host, False
 
 
-def url_fix_host (urlparts):
+def split_netloc(netloc):
+    """Separate userinfo from host in urllib.parse.SplitResult.netloc.
+    Originated as urllib.parse._splituser().
+    """
+    userinfo, delim, hostport = netloc.rpartition('@')
+    return (userinfo if delim else None), hostport
+
+
+def url_fix_host(urlparts, encoding):
     """Unquote and fix hostname. Returns is_idn."""
     if not urlparts[1]:
-        urlparts[2] = parse.unquote(urlparts[2])
+        urlparts[2] = urllib.parse.unquote(urlparts[2], encoding=encoding)
         return False
-    userpass, netloc = parse.splituser(urlparts[1])
+    userpass, hostport = split_netloc(urlparts[1])
     if userpass:
-        userpass = parse.unquote(userpass)
-    netloc, is_idn = idna_encode(parse.unquote(netloc).lower())
+        userpass = urllib.parse.unquote(userpass, encoding=encoding)
+    netloc, is_idn = idna_encode(
+        urllib.parse.unquote(hostport, encoding=encoding).lower()
+    )
     # a leading backslash in path causes urlsplit() to add the
     # path components up to the first slash to host
     # try to find this case...
     i = netloc.find("\\")
     if i != -1:
         # ...and fix it by prepending the misplaced components to the path
-        comps = netloc[i:] # note: still has leading backslash
+        comps = netloc[i:]  # note: still has leading backslash
         if not urlparts[2] or urlparts[2] == '/':
             urlparts[2] = comps
         else:
-            urlparts[2] = "%s%s" % (comps, parse.unquote(urlparts[2]))
+            urlparts[2] = "%s%s" % (
+                comps,
+                urllib.parse.unquote(urlparts[2], encoding=encoding),
+            )
         netloc = netloc[:i]
     else:
         # a leading ? in path causes urlsplit() to add the query to the
@@ -216,7 +207,7 @@ def url_fix_host (urlparts):
         if i != -1:
             netloc, urlparts[3] = netloc.split('?', 1)
         # path
-        urlparts[2] = parse.unquote(urlparts[2])
+        urlparts[2] = urllib.parse.unquote(urlparts[2], encoding=encoding)
     if userpass:
         # append AT for easy concatenation
         userpass += "@"
@@ -230,98 +221,80 @@ def url_fix_host (urlparts):
         if port != dport:
             host = "%s:%d" % (host, port)
         netloc = host
-    urlparts[1] = userpass+netloc
+    urlparts[1] = userpass + netloc
     return is_idn
 
 
-def url_fix_common_typos (url):
-    """Fix common typos in given URL like forgotten colon."""
-    if url.startswith("http//"):
-        url = "http://" + url[6:]
-    elif url.startswith("https//"):
-        url = "https://" + url[7:]
-    return url
-
-
-def url_fix_mailto_urlsplit (urlparts):
+def url_fix_mailto_urlsplit(urlparts):
     """Split query part of mailto url if found."""
-    if "?" in urlparts[2]:
-        urlparts[2], urlparts[3] = urlparts[2].split('?', 1)
+    sep = "?"
+    if sep in urlparts[2]:
+        urlparts[2], urlparts[3] = urlparts[2].split(sep, 1)
+
 
 # wayback urls include in the path http[s]://. By default the
 # tidying mechanism in linkchecker encodes the : and deletes the second slash
 # This function reverses these corrections. This function expects only the
 # path section of the URL as input.
 wayback_regex = re.compile(r'(https?)(\%3A/|:/)')
+
+
 def url_fix_wayback_query(path):
     return wayback_regex.sub(r'\1://', path)
 
-def url_parse_query (query, encoding=None):
+
+def url_parse_query(query, encoding):
     """Parse and re-join the given CGI query."""
-    if isinstance(query, unicode):
-        if encoding is None:
-            encoding = url_encoding
-        query = query.encode(encoding, 'ignore')
     # if ? is in the query, split it off, seen at msdn.microsoft.com
     append = ""
     while '?' in query:
         query, rest = query.rsplit('?', 1)
-        append = '?'+url_parse_query(rest)+append
-    l = []
-    for k, v, sep in parse_qsl(query, keep_blank_values=True):
-        k = url_quote_part(k, '/-:,;')
+        append = '?' + url_parse_query(rest, encoding=encoding) + append
+    f = []
+    for k, v, sep in parse_qsl(query, keep_blank_values=True, encoding=encoding):
+        k = urllib.parse.quote(k, safe='/-:,;')
         if v:
-            v = url_quote_part(v, '/-:,;')
-            l.append("%s=%s%s" % (k, v, sep))
+            v = urllib.parse.quote(v, safe='/-:,;')
+            f.append("%s=%s%s" % (k, v, sep))
         elif v is None:
-            l.append("%s%s" % (k, sep))
+            f.append("%s%s" % (k, sep))
         else:
             # some sites do not work when the equal sign is missing
-            l.append("%s=%s" % (k, sep))
-    return ''.join(l) + append
+            f.append("%s=%s" % (k, sep))
+    return ''.join(f) + append
 
 
-def urlunsplit (urlparts):
-    """Same as urlparse.urlunsplit but with extra UNC path handling
+def urlunsplit(urlparts):
+    """Same as urllib.parse.urlunsplit but with extra UNC path handling
     for Windows OS."""
-    res = urlparse.urlunsplit(urlparts)
+    res = urllib.parse.urlunsplit(urlparts)
     if os.name == 'nt' and urlparts[0] == 'file' and '|' not in urlparts[2]:
         # UNC paths must have 4 slashes: 'file:////server/path'
-        # Depending on the path in urlparts[2], urlparse.urlunsplit()
+        # Depending on the path in urlparts[2], urllib.parse.urlunsplit()
         # left only two or three slashes. This is fixed below
         repl = 'file://' if urlparts[2].startswith('//') else 'file:/'
         res = res.replace('file:', repl)
     return res
 
 
-def url_norm (url, encoding=None):
+def url_norm(url, encoding):
     """Normalize the given URL which must be quoted. Supports unicode
     hostnames (IDNA encoding) according to RFC 3490.
 
     @return: (normed url, idna flag)
     @rtype: tuple of length two
     """
-    if isinstance(url, unicode):
-        # try to decode the URL to ascii since urllib.unquote()
-        # handles non-unicode strings differently
-        try:
-            url = url.encode('ascii')
-        except UnicodeEncodeError:
-            pass
-        encode_unicode = True
-    else:
-        encode_unicode = False
-    urlparts = list(urlparse.urlsplit(url))
+    urlparts = list(urllib.parse.urlsplit(url))
     # scheme
-    urlparts[0] = parse.unquote(urlparts[0]).lower()
+    urlparts[0] = urllib.parse.unquote(urlparts[0], encoding=encoding).lower()
     # mailto: urlsplit is broken
     if urlparts[0] == 'mailto':
         url_fix_mailto_urlsplit(urlparts)
     # host (with path or query side effects)
-    is_idn = url_fix_host(urlparts)
+    is_idn = url_fix_host(urlparts, encoding)
     # query
     urlparts[3] = url_parse_query(urlparts[3], encoding=encoding)
-    if urlparts[0] in urlparse.uses_relative:
+    if urlparts[0] in urllib.parse.uses_relative:
         # URL has a hierarchical path we should norm
         if not urlparts[2]:
             # Empty path is allowed if both query and fragment are also empty.
@@ -333,20 +306,19 @@ def url_norm (url, encoding=None):
             # fix redundant path parts
             urlparts[2] = collapse_segments(urlparts[2])
     # anchor
-    urlparts[4] = parse.unquote(urlparts[4])
+    urlparts[4] = urllib.parse.unquote(urlparts[4], encoding=encoding)
     # quote parts again
-    urlparts[0] = url_quote_part(urlparts[0], encoding=encoding) # scheme
-    urlparts[1] = url_quote_part(urlparts[1], safechars='@:', encoding=encoding) # host
-    urlparts[2] = url_quote_part(urlparts[2], safechars=_nopathquote_chars, encoding=encoding) # path
+    urlparts[0] = urllib.parse.quote(urlparts[0])  # scheme
+    urlparts[1] = urllib.parse.quote(urlparts[1], safe='@:')  # host
+    urlparts[2] = urllib.parse.quote(urlparts[2], safe=_nopathquote_chars)  # path
     if not urlparts[0].startswith("feed"):
-        urlparts[2] = url_fix_wayback_query(urlparts[2]) # unencode colon in http[s]:// in wayback path
-    urlparts[4] = url_quote_part(urlparts[4], safechars="!$&'()*+,-./;=?@_~", encoding=encoding) # anchor
+        # unencode colon in http[s]:// in wayback path
+        urlparts[2] = url_fix_wayback_query(urlparts[2])
+    urlparts[4] = urllib.parse.quote(urlparts[4], safe="!$&'()*+,-./;=?@_~")  # anchor
     res = urlunsplit(urlparts)
     if url.endswith('#') and not urlparts[4]:
         # re-append trailing empty fragment
         res += '#'
-    if encode_unicode:
-        res = unicode(res)
     return (res, is_idn)
 
 
@@ -355,7 +327,9 @@ _thisdir_ro = re.compile(r"^\./")
 _samedir_ro = re.compile(r"/\./|/\.$")
 _parentdir_ro = re.compile(r"^/(\.\./)+|/(?!\.\./)[^/]+/\.\.(/|$)")
 _relparentdir_ro = re.compile(r"^(?!\.\./)[^/]+/\.\.(/|$)")
-def collapse_segments (path):
+
+
+def collapse_segments(path):
     """Remove all redundant segments from the given URL path.
     Precondition: path is an unquoted url path"""
     # replace backslashes
@@ -391,73 +365,50 @@ def collapse_segments (path):
 url_is_absolute = re.compile(r"^[-\.a-z]+:", re.I).match
 
 
-def url_quote (url):
+def url_quote(url, encoding):
     """Quote given URL."""
     if not url_is_absolute(url):
         return document_quote(url)
-    urlparts = list(urlparse.urlsplit(url))
-    urlparts[0] = url_quote_part(urlparts[0]) # scheme
-    urlparts[1] = url_quote_part(urlparts[1], ':') # host
-    urlparts[2] = url_quote_part(urlparts[2], '/=,') # path
-    urlparts[3] = url_quote_part(urlparts[3], '&=,') # query
-    l = []
-    for k, v, sep in parse_qsl(urlparts[3], True): # query
-        k = url_quote_part(k, '/-:,;')
+    urlparts = list(urllib.parse.urlsplit(url))
+    urlparts[0] = urllib.parse.quote(urlparts[0])  # scheme
+    urlparts[1] = urllib.parse.quote(urlparts[1], safe=':')  # host
+    urlparts[2] = urllib.parse.quote(urlparts[2], safe='/=,')  # path
+    urlparts[3] = urllib.parse.quote(urlparts[3], safe='&=,')  # query
+    f = []
+    for k, v, sep in parse_qsl(
+        urlparts[3], encoding=encoding, keep_blank_values=True
+    ):  # query
+        k = urllib.parse.quote(k, safe='/-:,;')
         if v:
-            v = url_quote_part(v, '/-:,;')
-            l.append("%s=%s%s" % (k, v, sep))
+            v = urllib.parse.quote(v, safe='/-:,;')
+            f.append("%s=%s%s" % (k, v, sep))
         else:
-            l.append("%s%s" % (k, sep))
-    urlparts[3] = ''.join(l)
-    urlparts[4] = url_quote_part(urlparts[4]) # anchor
+            f.append("%s%s" % (k, sep))
+    urlparts[3] = ''.join(f)
+    urlparts[4] = urllib.parse.quote(urlparts[4])  # anchor
     return urlunsplit(urlparts)
 
 
-def url_quote_part (s, safechars='/', encoding=None):
-    """Wrap urllib.quote() to support unicode strings. A unicode string
-    is first converted to UTF-8. After that urllib.quote() is called."""
-    if isinstance(s, unicode):
-        if encoding is None:
-            encoding = url_encoding
-        s = s.encode(encoding, 'ignore')
-    return parse.quote(s, safechars)
-
-def document_quote (document):
+def document_quote(document):
     """Quote given document."""
-    doc, query = parse.splitquery(document)
-    doc = url_quote_part(doc, '/=,')
+    doc, delim, query = document.rpartition('?')
+    if not delim:
+        doc = document
+        query = None
+    doc = urllib.parse.quote(doc, safe='/=,')
     if query:
         return "%s?%s" % (doc, query)
     return doc
 
 
-def match_url (url, domainlist):
-    """Return True if host part of url matches an entry in given domain list.
-    """
-    if not url:
-        return False
-    return match_host(url_split(url)[1], domainlist)
-
-
-def match_host (host, domainlist):
-    """Return True if host matches an entry in given domain list."""
-    if not host:
-        return False
-    for domain in domainlist:
-        if domain.startswith('.'):
-            if host.endswith(domain):
-                return True
-        elif host == domain:
-            return True
-    return False
-
-
 _nopathquote_chars = "-;/=,~*+()@!"
 if os.name == 'nt':
     _nopathquote_chars += "|"
-_safe_url_chars = re.escape(_nopathquote_chars + "_:.&#%?[]!")+"a-zA-Z0-9"
+_safe_url_chars = re.escape(_nopathquote_chars + "_:.&#%?[]!") + "a-zA-Z0-9"
 _safe_url_chars_ro = re.compile(r"^[%s]*$" % _safe_url_chars)
-def url_needs_quoting (url):
+
+
+def url_needs_quoting(url):
     """Check if url needs percent quoting. Note that the method does
     only check basic character sets, and not any other syntax.
     The URL might still be syntactically incorrect even when
@@ -470,35 +421,14 @@ def url_needs_quoting (url):
     return not _safe_url_chars_ro.match(url)
 
 
-def url_split (url):
-    """Split url in a tuple (scheme, hostname, port, document) where
-    hostname is always lowercased.
-    Precondition: url is syntactically correct URI (eg has no whitespace)
-    """
-    scheme, netloc = parse.splittype(url)
-    host, document = parse.splithost(netloc)
-    port = default_ports.get(scheme, 0)
-    if host:
-        host = host.lower()
-        host, port = splitport(host, port=port)
-    return scheme, host, port, document
-
-
-def url_unsplit (parts):
-    """Rejoin URL parts to a string."""
-    if parts[2] == default_ports.get(parts[0]):
-        return "%s://%s%s" % (parts[0], parts[1], parts[3])
-    return "%s://%s:%d%s" % parts
-
-
-def splitport (host, port=0):
+def splitport(host, port=0):
     """Split optional port number from host. If host has no port number,
     the given default port is returned.
 
     @param host: host name
-    @ptype host: string
+    @type host: string
     @param port: the port number (default 0)
-    @ptype port: int
+    @type port: int
 
     @return: tuple of (host, port)
     @rtype: tuple of (string, int)
@@ -515,69 +445,3 @@ def splitport (host, port=0):
             # For an invalid non-empty port leave the host name as is
             pass
     return host, port
-
-
-def get_content(url, user=None, password=None, proxy=None, data=None,
-                addheaders=None):
-    """Get URL content and info.
-
-    @return: (decoded text content of URL, headers) or
-             (None, errmsg) on error.
-    @rtype: tuple (String, dict) or (None, String)
-    """
-    from . import configuration
-    headers = {
-        'User-Agent': configuration.UserAgent,
-    }
-    if addheaders:
-        headers.update(addheaders)
-    method = 'GET'
-    kwargs = dict(headers=headers)
-    if user and password:
-        kwargs['auth'] = (user, password)
-    if data:
-        kwargs['data'] = data
-        method = 'POST'
-    if proxy:
-        kwargs['proxy'] = dict(http=proxy)
-    from .configuration import get_share_file
-    try:
-        kwargs["verify"] = get_share_file('cacert.pem')
-    except ValueError:
-        pass
-    try:
-        response = requests.request(method, url, **kwargs)
-        return response.text, response.headers
-    except (requests.exceptions.RequestException,
-            requests.exceptions.BaseHTTPError) as msg:
-        log.warn(LOG_CHECK, ("Could not get content of URL %(url)s: %(msg)s.") \
-          % {"url": url, "msg": str(msg)})
-        return None, str(msg)
-
-
-def shorten_duplicate_content_url(url):
-    """Remove anchor part and trailing index.html from URL."""
-    if '#' in url:
-        url = url.split('#', 1)[0]
-    if url.endswith('index.html'):
-        return url[:-10]
-    if url.endswith('index.htm'):
-        return url[:-9]
-    return url
-
-
-def is_duplicate_content_url(url1, url2):
-    """Check if both URLs are allowed to point to the same content."""
-    if url1 == url2:
-        return True
-    if url2 in url1:
-        url1 = shorten_duplicate_content_url(url1)
-        if not url2.endswith('/') and url1.endswith('/'):
-            url2 += '/'
-        return url1 == url2
-    if url1 in url2:
-        url2 = shorten_duplicate_content_url(url2)
-        if not url1.endswith('/') and url2.endswith('/'):
-            url1 += '/'
-        return url1 == url2
-    return False
